<?php
/**
 * NOTICE OF LICENSE.
 *
 * This source file is subject to the following license: REGULAR LICENSE
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade to newer
 * versions in the future.
 *
 * @author    Envía Ya S.A. de C.V
 * @copyright Envía Ya S.A. de C.V https://enviaya.com.mx
 * @license   REGULAR LICENSE
 */

namespace EnviaYa\WooCommerce\Shipping;

defined('ABSPATH') || exit;

if (!class_exists(__NAMESPACE__ . '\\Plugin')):

class Plugin
{
	protected $id;
	protected $main_menu_id;
	protected $title;
	protected $description;
	protected $option_key;
	protected $settings;
	protected $plugin_path;
	protected $version;
	protected $adapter;
	protected $logger;
	protected $page_detector;
	protected $cart_proxy;
	protected $session_proxy;

    public function __construct($plugin_path) 
    {
		$this->id = 'wc-enviaya-shipping';
		$this->title = 'EnvíaYa: Ship with all carriers (Rating, Booking and Tracking)';
		$this->description = 'An powerful plugin to rate multi-carrier shipment services and est. delivery dates during checkout, create shipment labels with one click and track.';
		$this->plugin_path = $plugin_path;
		$this->version = '1.0.10';
		$this->option_key = 'wc_enviaya_shipping_settings';
		$this->settings = array();
		$this->pluginSettings = array();

		$this->mainMenuId = 'EnviaYa';
		$this->adapter = null;
		$this->settingsFormHooks = null;
		$this->logger = &\EnviaYa\WooCommerce\Logger\LoggerInstance::getInstance($this->id); 
		$this->pageDetector = new \EnviaYa\WooCommerce\Utils\PageDetector();
		// initialize proxies so we will always have something to work with
		$this->cartProxy = new \EnviaYa\Proxies\LazyClassProxy('stdClass');
		$this->sessionProxy = new \EnviaYa\Proxies\LazyClassProxy('stdClass');
		$this->countryProxy = new \EnviaYa\Proxies\LazyClassProxy('stdClass');
	
	}

	protected function init_adapter()
	{
		$this->adapter = new \EnviaYa\WooCommerce\Shipping\Adapter\EnviaYa($this->id);		
	}


	public function register()
	{

		//echo '<pre>';var_dump($wp_filter);echo '</pre>';exit();

		if (!function_exists('is_plugin_active')) {
			require_once(ABSPATH . 'wp-admin/includes/plugin.php');
		}

		// do not register when WooCommerce is not enabled
		if (!is_plugin_active('woocommerce/woocommerce.php')) {
			return;
		}
		
		// activation hooks
		register_activation_hook( plugin_basename($this->plugin_path), array($this, 'onActivate') );
		register_deactivation_hook(  plugin_basename($this->plugin_path), array($this, 'onDesactivate') );

		

		$this->init_adapter();
		$this->load_settings();
		
		$this->loadPluginTextdomain();

		

		if (is_admin()) {
		
			\EnviaYa\WooCommerce\Admin\EnviaYaAdmin::instance()->register($this->id, $this->plugin_path);
			add_action('admin_menu', array($this, 'onAdminMenu'),10);
			
			add_action('admin_notices', array($this, 'onLoadAdminNotices'),10);
			add_action('wp_ajax_get_billing_accounts',  array($this, 'getBillingAccounts'));
			add_action('wp_ajax_get_origin_address',  array($this, 'onGetOriginAddress') );

			add_filter('woocommerce_get_sections_shipping', array( $this, 'onAddShippingSettingsSectionTab'),10 );

		}else{

			wp_register_style(
				'wc-enviaya-shipping-methods',
				plugins_url('assets/css/public/shipping-methods.css',$this->plugin_path),
				array(),
				$this->version,
				'all'
			);
			
			wp_enqueue_style( 'wc-enviaya-shipping-methods' );
			
			
			/*
			wp_register_style(
				'wc-enviaya-shipping-settings',
				plugins_url('assets/css/admin/settings.css',$this->plugin_path),
				array(),
				$this->version,
				'all'
			);
	
			if(get_current_screen()->id == 'woocommerce_page_wc-settings'){
				wp_enqueue_script( 'thickbox' );
				wp_enqueue_style( 'thickbox' );
				wp_enqueue_script( 'wc-enviaya-shipping-settings' );
				wp_enqueue_style( 'wc-enviaya-shipping-settings' );
				
			}
			*/

		}
		
		//add_action($this->id . '_check_origin_direction', array($this, 'onCheckOriginDirection') );
		add_filter('plugin_action_links_' . plugin_basename($this->plugin_path), array($this, 'onPluginActionLinks'), 1, 1);
		add_filter('woocommerce_shipping_methods', array($this, 'add_shipping_method'));
		add_filter($this->id . '_get_plugin_settings', array($this, 'get_plugin_settings'), 10, 1);
		add_filter($this->id . '_is_enabled', array($this, 'is_enabled'), 10, 1);
		add_filter($this->id . '_init_form_fields', array($this, 'updateFormFields'), 1, 1);
		add_filter($this->id . '_service_name', array($this, 'getDefaultServiceName'), 1, 2);
		add_filter($this->id . '_is_cart', array($this->pageDetector, 'is_cart'), 1, 0);
		add_filter($this->id . '_is_checkout', array($this->pageDetector, 'is_checkout'), 1, 0);
		add_action('plugins_loaded', array($this, 'initLazyClassProxies'), 1, 0);
		
		add_action('wp_loaded', array($this, 'calculateShippingOnCheckout'), 1, 0);
		add_action('woocommerce_after_checkout_validation', array($this, 'onCheckoutValidation'), PHP_INT_MAX, 2);
		add_filter('woocommerce_billing_fields', array($this, 'setRequiredFields'), 10, 1);
		add_filter('woocommerce_shipping_fields', array($this, 'setRequiredFields'), 10, 1);

		add_filter( 'woocommerce_shipping_package_name', array($this, 'setShippingTitle'), 10, 1);

		add_filter( 'woocommerce_cart_shipping_method_full_label', array($this, 'displayCarrierLogo'), 10, 2 ); 
		
		

	}

	public function displayCarrierLogo( $label, $method ) { 
		
		
		
		if($method->get_method_id() == 'wc-enviaya-shipping'){
			
			$rate_id = $method->get_meta_data()['rate_id'];
			
			if($rate_id != 'enviaya-local-pickup'){

				$carrier_logo_height_map = array(
					'99minutos' => '20px',
					'sendex' => '20px',
					'redpack' => '9px',
					'ampm' => '18px',
					'fedex' => '12px',
					'ups' => '25px',
					'ivoy' => '18px',
				);

				$carrier = strtolower($method->get_meta_data()['carrier_name']);
				
				$carrier_logo_height = isset($carrier_logo_height_map[$carrier]) ? $carrier_logo_height_map[$carrier] : '10px';
			
				if( isset($this->settings['display_carrier_logo']) && $this->settings['display_carrier_logo'] == 'yes'){
					$label = '<img class="ey-carrier-logo" src="'.$method->get_meta_data()['carrier_logo_url'].'" style="height:'.$carrier_logo_height.'; margin-right:5px; display:inline"/><span class="ey-label-text">'.$label.'</span>'; 
				}

			}

		}	
		
		return $label;
	}

	public function setShippingTitle( $name ) {

		if( isset($this->settings['service_title_name']) && !empty($this->settings['service_title_name']) ){
			return $this->settings['service_title_name'];
		}
	
		
	  }

	

	protected function load_settings()
	{		
		$this->init_settings();

		$this->settings = array_merge($this->settings, (array)get_option($this->option_key, array()));
		
		$this->logger->setEnabled($this->settings['debug'] == 'yes');

		$this->adapter->setSettings($this->settings);
	}

	protected function init_settings()
	{

		$this->settings = array(
			'test_mode' => 'no',
			'enabled' => 'yes',
			'fetch_rates_page_condition' => 'cart',
			'display_carrier_logo' => 'no',
			'display_carrier_name' => 'yes',
			'display_service_name' => 'dynamic_service_name',
			'display_delivery_time' => 'no_delivery_time',
			'enable_standard_flat_rate' => 'no',
			'contingency_standard_shipping_title' => __('Standard shipping', 'wc-enviaya-shipping'),
			'standard_flat_rate' => 100,
			'enable_express_flat_rate' => 'no',
			'contingency_express_shipping_title' => __('Express Flat Rate', 'wc-enviaya-shipping'),
			'express_flat_rate' => 150,
			'automatic_booking_shipment' => 'no' ,
			'enable_dokan_integration' => 'no',
			'enable_shipping_zones' => 'no',
			'debug' => 'no',
			'cache' => 'yes',
			'cache_expiration_in_secs' => 12 * 60 * 60,
			'timeout' => 10,
			'remove_settings' => 'no',
		);
		
	}

	public function get_plugin_settings()
	{
		return $this->settings;
	}

	public function onActivate()
	{

		wp_schedule_event(time(), 'daily', $this->id . '_check_origin_direction');

		$old_settings = get_option('woocommerce_enviaya_settings');
		$new_settings = get_option('wc_enviaya_shipping_settings');
		if($old_settings){
			
			
			if(isset($old_settings['api_key_production']) && !empty($old_settings['api_key_production'])){
				$new_settings['production_api_key'] = $old_settings['api_key_production'];
			}

			if(isset($old_settings['api_key_test']) && !empty($old_settings['api_key_test'])){
				$new_settings['test_api_key'] = $old_settings['api_key_test'];
			}

			if(isset($old_settings['enabled_test_mode']) && !empty($old_settings['enabled_test_mode'])){
				$new_settings['test_mode'] = $old_settings['enabled_test_mode'];
			}

			if(isset($old_settings['enviaya_account']) && !empty($old_settings['enviaya_account'])){
				$new_settings['account'] = $old_settings['enviaya_account'];
			}

			if(isset($old_settings['account_id']) && !empty($old_settings['account_id'])){
				$new_settings['account_id'] = $old_settings['account_id'];
			}

			if(isset($old_settings['origin_address']) && !empty($old_settings['origin_address'])){
				
				$origin_address = json_decode(str_replace('||', '"', $old_settings['origin_address']),true);
				foreach($origin_address as $k => $v){

					if(!is_null($v) && !empty($v)){
						$key = 'origin_direction_'.$k;
						$new_settings[$key] = $v;
					
					}
					
				}		

			}

			update_option('wc_enviaya_shipping_settings', $new_settings);
			delete_option('woocommerce_enviaya_settings');

		}
	}

	public function onDesactivate(){
		
		if (wp_next_scheduled ( $this->id . '_check_origin_direction' )) { 
            wp_clear_scheduled_hook( $this->id . '_check_origin_direction' );
        }

		if( isset($this->settings['remove_settings']) && $this->settings['remove_settings'] == 'yes' ){

			delete_option('wc_enviaya_shipping_settings');
			delete_option('woocommerce_enviaya_settings');

			

		}

		

	}

	public function onLoadAdminNotices(){
		
		if( isset($this->settings['enviaya_change_origin_direction']) && is_null( $_COOKIE['ENVIAYA_NO_UPDATE_ORIGIN_ADDRESS'] ) ){
			if( $this->settings['enviaya_change_origin_direction']['notice'] == 'yes' ){

				$update_url = wp_nonce_url(
					add_query_arg( array('do_update_origin_address' => 'true', 'subtab' => 'sender' ), admin_url( 'admin.php?page=wc-settings&tab=shipping&section=' . $this->id ) ),
					'update_origin_address_nonce',
				);

				$contact_keys = ['title', 'phone', 'email'];
				$direction_keys = ['direction_1', 'direction_2', 'neighborhood', 'district', 'postal_code', 'city', 'state_code', 'country_code'];
				
				$current_woo_direction_arr = json_decode($this->settings['enviaya_change_origin_direction']['current_woo_direction'],true);

				$current_woo_direction_arr['title'] = (!is_null($current_woo_direction_arr['company'])) ? $current_woo_direction_arr['full_name']." ( ".$current_woo_direction_arr['company']." )" : $current_woo_direction_arr['full_name']; 

				$current_enviaya_direction_arr = json_decode($this->settings['enviaya_change_origin_direction']['current_enviaya_direction'],true);
				
				$current_enviaya_direction_arr['title'] = (!is_null($current_enviaya_direction_arr['company'])) ? $current_enviaya_direction_arr['full_name']." ( ".$current_enviaya_direction_arr['company']." )" : $current_enviaya_direction_arr['full_name']; 

				$current_woo_direction_txt = "";
				
				$current_enviaya_direction_txt = "";

				foreach($contact_keys as $k){

					if( isset($current_woo_direction_arr[$k]) && !is_null($current_woo_direction_arr[$k]) ){

						$current_woo_direction_txt.=$current_woo_direction_arr[$k];
						$current_woo_direction_txt.="<br/>"; 
      
					}

					if( isset($current_enviaya_direction_arr[$k]) && !is_null($current_enviaya_direction_arr[$k]) ){

						$current_enviaya_direction_txt.=$current_enviaya_direction_arr[$k];
						$current_enviaya_direction_txt.="<br/>"; 
      
					}
				}

				$current_woo_direction_txt.="<br/>"; 
				$current_enviaya_direction_txt.="<br/>"; 

				foreach($direction_keys as $k){

					if( isset($current_woo_direction_arr[$k]) && !is_null($current_woo_direction_arr[$k]) ){
						$current_woo_direction_txt.=$current_woo_direction_arr[$k];
						
						if($k == 'postal_code' ){
							$current_woo_direction_txt.=" "; 
						}elseif($k == 'city'){
							$current_woo_direction_txt.=", "; 
						}else{
							$current_woo_direction_txt.="<br/>"; 
						}
      
					}

					if( isset($current_enviaya_direction_arr[$k]) && !is_null($current_enviaya_direction_arr[$k]) ){
						$current_enviaya_direction_txt.=$current_enviaya_direction_arr[$k];
						
						if($k == 'postal_code' ){
							$current_enviaya_direction_txt.=" "; 
						}elseif($k == 'city'){
							$current_enviaya_direction_txt.=", "; 
						}else{
							$current_enviaya_direction_txt.="<br/>"; 
						}
      
					}

				}
				
				$nonce =  wp_create_nonce('ajax-nonce');

				echo    esc_html('<div class="notice notice-warning is-dismissible" id="origin_direction_change_notice">
							<p style="display:flex; align-items:center"><img src="'.plugins_url('assets/images/enviaya-icon.png', dirname(dirname(str_replace('phar://', '', __FILE__)))).'"><strong style="margin-left:10px"> '.__('The origin direction has changed from EnvíaYa account','wc-enviaya-shipping').'</strong></p>
							<p>'.__('The origin direction has changed from EnvíaYa account and It is different to origin direction configured on plugin.','wc-enviaya-shipping').'</p>
							<p><strong>'.__('Current origin direction on plugin','wc-enviaya-shipping').':</strong></p>
							<p>'.strtoupper($current_woo_direction_txt).'</p>
							<p><strong>'.__('Current origin direction on EnvíaYa app','wc-enviaya-shipping').':</strong></p>
							<p>'.strtoupper($current_enviaya_direction_txt).'</p>
							<p>'.__('Do you want to update the origin direction configured on plugin?','wc-enviaya-shipping').'</p>
							<p class="submit">
								<a href="'. esc_url($update_url) .'" class="wc-update-now button-primary">'.__('Yes, update','wc-enviaya-shipping').'</a>
								<a onclick="myFunction()" href="#no_update" class="button-secondary">'.__('No, no update','wc-enviaya-shipping').'</a>
							</p>
						</div>
						<script>
							function myFunction() {
								jQuery.post("'.admin_url('admin-ajax.php').'", {action:"do_no_update_origin_address", nonce:"'.$nonce.'"}).done(function( res ) { jQuery("#origin_direction_change_notice").remove()  })
							}
						</script>');

						
			}
		}
		
	}

	public function setRequiredFields($fields)
	{
		if ('yes' !== $this->settings['requireCompanyName']) {
			return $fields;
		}
		
		if (isset($fields['billing_company'])) {
			$fields['billing_company']['required'] = true;
		}

		if (isset($fields['shipping_company'])) {
			$fields['shipping_company']['required'] = true;
		}

		return $fields;
	}

	/*
	public function onCheckOriginDirection(){

		if( is_null( $_COOKIE['ENVIAYA_NO_UPDATE_ORIGIN_ADDRESS'] ) && isset($this->settings['enviaya_origin_direction']) && json_decode($this->settings['enviaya_origin_direction'],true) ){

			$origin_direction_wp = json_decode($this->settings['enviaya_origin_direction'],true);

			if( isset($origin_direction_wp['id']) && !empty($origin_direction_wp['id']) ){
				
				$id = $origin_direction_wp['id'];

				$result = $this->adapter->getOriginDirectionById($id,array(
					'api_key' => $this->adapter->getApiKey(),
				));

				if (!empty($result['error']['message'])) {
					return false;
				}			
				
				$direction = $result['response'];

				$flag = false;

				foreach($direction as $k => $v){

					if( $origin_direction_wp[$k] != $v ){
						$flag = true;
					}
					
				}

				if($flag){

					$settings = $this->settings;
					$settings['enviaya_change_origin_direction'] = array(
						'notice' => 'yes',
						'current_woo_direction' => json_encode($origin_direction_wp),
						'current_enviaya_direction' => json_encode($direction),
					);

					$settings = apply_filters('woocommerce_settings_api_sanitized_fields_' . $this->id, $settings);
					
					update_option('wc_enviaya_shipping_settings', $settings);


				}

			}
			
		}
		
	}
	*/

	public function onCheckoutValidation($postedData, $checkoutErrors)
	{		
		if ($this->settings['validateAddress'] != 'yes') {
			return;
		}
		
		$validationErrors = $this->sessionProxy->get($this->id . '_validationErrors');
		if (empty($validationErrors)) {
			$validationErrors = array();
		}

		$this->logger->debug(__FILE__, __LINE__, 'onCheckoutValidation: ' . print_r($validationErrors, true));

		foreach ($validationErrors as $fieldKey => $errors) {
			$errorPrefix = '';
			if ($fieldKey == 'origin') {
				$errorPrefix = __('From Address:', 'wc-enviaya-shipping');
			} else if ($fieldKey == 'destination') {
				$errorPrefix = __('Shipping Address:', 'wc-enviaya-shipping');
			}

			foreach ($errors as $idx => $error) {
				$checkoutErrors->add($this->id . '_validation_error_' . $idx, sprintf('<strong>%s</strong> %s', $errorPrefix, $error));
			}
		}
	}

	public function onAdminMenu()
	{
		
		add_menu_page(
			'EnvíaYa! for WooCommerce',
			'EnvíaYa!',
			'manage_options',
			'admin.php?page=wc-settings&tab=shipping&section=' . $this->id,
			'',
			plugins_url('assets/images/enviaya-icon.png', dirname(dirname(str_replace('phar://', '', __FILE__)))),
			26
		);

		add_submenu_page('woocommerce', 'EnvíaYa!', 'EnvíaYa!', 'manage_options', 'admin.php?page=wc-settings&amp;tab=shipping&amp;section=wc-enviaya-shipping');

	}

	

	public function onPluginActionLinks($links)
	{
		$link = sprintf('<a href="%s">%s</a>', admin_url('admin.php?page=wc-settings&tab=shipping&section=' . $this->id), __('Settings', 'wc-enviaya-shipping'));
		array_unshift($links, $link);
		return $links;
	}


	public function updateFormFields($formFields)
	{
		
		return $this->adapter->updateFormFields($formFields);
	}

	public function getDefaultServiceName($name, $service)
	{
		if (empty($name)) {
			$services = $this->adapter->getServices();
			if (!empty($services[$service])) {
				$name = $services[$service];
			}
		}

		return $name;
	}

	public function initLazyClassProxies()
	{
		// it can't work in the ADMIN or when WC is undefined
		if (!function_exists('WC')) {
			return;
		}

		if(is_admin()){
			$this->countryProxy = new \EnviaYa\Proxies\LazyClassProxy('WC_Countries');
		}else{		
			$this->cartProxy = new \EnviaYa\Proxies\LazyClassProxy('WC_Cart', WC()->cart);
			$this->sessionProxy = new \EnviaYa\Proxies\LazyClassProxy(apply_filters('woocommerce_session_handler', 'WC_Session_Handler'), WC()->session);
		}
		
	}

	public function calculateShippingOnCheckout()
	{
		if ($this->settings['fetchRatesPageCondition'] != 'checkout') {
			return;
		}

		if (!apply_filters($this->id . '_is_checkout', false)) {
			$this->sessionProxy->set($this->id . '_' . __FUNCTION__, false);
			return;
		}

		$this->logger->debug(__FILE__, __LINE__,  __FUNCTION__);

		$packages = $this->cartProxy->get_shipping_packages();
		if (is_array($packages) && !empty($packages) && !$this->sessionProxy->get($this->id . '_' . __FUNCTION__)) {
			foreach ($packages as $packageKey => $package) {
				$sessionKey = 'shipping_for_package_' . $packageKey;
				$this->sessionProxy->set($sessionKey, null);
			}
	
			$this->cartProxy->calculate_shipping();
			$this->cartProxy->calculate_totals();	

			$this->sessionProxy->set($this->id . '_' . __FUNCTION__, true);
		}
	}

	public function loadPluginTextdomain(){

		$locale = apply_filters( 'plugin_locale', determine_locale(), $this->id );
		
		load_plugin_textdomain('wc-enviaya-shipping', false, 'wc-enviaya-shipping/languages/');
		
	}

    public function add_shipping_method($methods)
	{
		
		$methods[$this->id] = '\EnviaYa\WooCommerce\Shipping\EnviaYaShippingMethod';
		
		return $methods;
	}

	public function is_enabled($enabled = false)
	{
		
		if ($this->settings['enabled'] == 'yes') {
			$enabled = true;
		}

		return $enabled;
	}

	public function onAddShippingSettingsSectionTab( $section )
	{

		$section['wc-enviaya-shipping'] = 'EnvíaYa!';
		
		return $section;
	}

	public function onGetOriginAddress(){

		$action = sanitize_text_field($_POST['action']);

		$api_key = isset($_POST['api_key']) ? sanitize_text_field($_POST['api_key']) : '';
		
		if($action === 'get_origin_address' && !empty($api_key) ) {

			$params = array(
				'api_key' => $api_key,
				'get_destinations' => 'false',
			);

			$full_name = isset($_POST['full_name']) ? sanitize_text_field($_POST['full_name']) : '';

			if(!empty($full_name)){
				$params['full_name'] = $full_name;
			}else{
				$params['per_page'] = '50';
			}
			
			$result = $this->adapter->getOriginDirections($params);
			
			if (!empty($result['error']['message'])) {
				wp_send_json_error($result['error']['message'],$result['error']['code']) ;
			}			
			
			if (!isset($result['response']['directions'])) {
				$this->logger->debug(__FILE__, __LINE__, 'No origin directions have been found');
				
				return null;
			}

			$directions = $result['response']['directions'];
			
			$this->logger->debug(__FILE__, __LINE__, 'directions: ' . print_r($directions, true));
			
			wp_send_json_success($directions);
			
		}
		
	}

	public function getBillingAccounts(){

		
		$action = sanitize_text_field($_POST['action']);
		
		if($action == 'get_billing_accounts' ) {
			
			$api_key = sanitize_text_field($_POST['api_key']);

			$result = $this->adapter->getBillingAccounts(array(
				'api_key' => $api_key,
				'function' => __FUNCTION__
			));
			
			
			if (!empty($result['error']['message'])) {
				wp_send_json_error($result['error']['message'],400) ;
			}			
			
			$accounts = $result['response']['enviaya_accounts'];
		
			$this->logger->debug(__FILE__, __LINE__, 'accounts: ' . print_r($accounts, true));
			
			wp_send_json_success($accounts);
		}
	}

};


endif;

